//
//  AngleView.swift
//  Winkel
//
//  Created by Heiko Etzold on 22.01.18.
//  Copyright © 2018 Heiko Etzold. All rights reserved.
//

import UIKit





class AngleView: UIView {
    
    var offsetAngle = CGFloat(0)
    var sizeOfAngle = CGFloat(30)
    var sizeOfCompareAngle = CGFloat(30)

    var lengthOfLine = CGFloat(200)
    
    var touchPoint : TouchPointForAngleView!
            
    var colorOfLines = labelColor
    var colorOfComparableLines = labelColor
    
    override func draw(_ rect: CGRect) {
        
        let arcDistance = lengthOfLine/2
        let arrowAngle = min(8,sizeOfAngle)
        let arrowLength = CGFloat(5)*arrowAngle/8

        //angle
        let center = self.convert(self.center, from: self.superview)
        let angleLines = UIGraphicsGetCurrentContext()!
        angleLines.move(to: center+CGPoint(x: lengthOfLine*cos(offsetAngle*(.pi)/180), y: -lengthOfLine*sin(offsetAngle*(.pi)/180)))
        angleLines.addLine(to: center)
        angleLines.addLine(to: center+CGPoint(x: lengthOfLine*cos((offsetAngle+sizeOfAngle)*(.pi)/180), y: -lengthOfLine*sin((offsetAngle+sizeOfAngle)*(.pi)/180)))
        
        //arc
        angleLines.move(to: center+CGPoint(x: arcDistance*cos(offsetAngle*(.pi)/180), y: -arcDistance*sin(offsetAngle*(.pi)/180)))
        if(arrowAngle != sizeOfAngle){
            angleLines.addArc(center: center, radius: arcDistance, startAngle: -offsetAngle*(.pi)/180, endAngle: (-offsetAngle-sizeOfAngle+arrowAngle)*(.pi)/180, clockwise: true)
        }
        
        //draw
        angleLines.setLineWidth(3)
        angleLines.setLineCap(.round)
        angleLines.setLineJoin(.round)
        angleLines.setStrokeColor(colorOfLines.cgColor)
        angleLines.strokePath()
        
        
        //arrow
        let angleArrow = UIGraphicsGetCurrentContext()!
        angleArrow.move(to: center+CGPoint(x: (arcDistance-arrowLength)*cos((offsetAngle+sizeOfAngle-arrowAngle)*(.pi)/180), y: -(arcDistance-arrowLength)*sin((offsetAngle+sizeOfAngle-arrowAngle)*(.pi)/180)))
        angleArrow.addLine(to: center+CGPoint(x: (arcDistance+arrowLength)*cos((offsetAngle+sizeOfAngle-arrowAngle)*(.pi)/180), y: -(arcDistance+arrowLength)*sin((offsetAngle+sizeOfAngle-arrowAngle)*(.pi)/180)))
        angleArrow.addLine(to: center+CGPoint(x: (arcDistance)*cos((offsetAngle+sizeOfAngle)*(.pi)/180), y: -(arcDistance)*sin((offsetAngle+sizeOfAngle)*(.pi)/180)))
        angleArrow.closePath()
        angleArrow.setLineJoin(.round)
        angleArrow.setFillColor(colorOfLines.cgColor)
        angleArrow.fillPath()

        
        //draw comparable angle
        if(compareAngleIsDrawn == true){
            
            let compareArcDistance = lengthOfLine/3
            let compareArrowAngle = min(8,sizeOfCompareAngle)
            let compareArrowLength = CGFloat(4)*compareArrowAngle/8

            //angle
            let compareAngleLines = UIGraphicsGetCurrentContext()!
            compareAngleLines.move(to: center+CGPoint(x: lengthOfLine*cos(offsetAngle*(.pi)/180), y: -lengthOfLine*sin(offsetAngle*(.pi)/180)))
            compareAngleLines.addLine(to: center)
            compareAngleLines.addLine(to: center+CGPoint(x: lengthOfLine*cos((offsetAngle+sizeOfCompareAngle)*(.pi)/180), y: -lengthOfLine*sin((offsetAngle+sizeOfCompareAngle)*(.pi)/180)))
            
            //arc
            compareAngleLines.move(to: center+CGPoint(x: lengthOfLine/2*cos(offsetAngle*(.pi)/180), y: -lengthOfLine/2*sin(offsetAngle*(.pi)/180)))
            if(compareArrowAngle != sizeOfCompareAngle){
                compareAngleLines.addArc(center: center, radius: compareArcDistance, startAngle: -offsetAngle*(.pi)/180, endAngle: (-offsetAngle-sizeOfCompareAngle+compareArrowAngle)*(.pi)/180, clockwise: true)
            }
            compareAngleLines.setLineWidth(2)
            compareAngleLines.setStrokeColor(colorOfComparableLines.cgColor)
            compareAngleLines.strokePath()
            
            //arrow
            let compareAngleArrow = UIGraphicsGetCurrentContext()!
            compareAngleArrow.move(to: center+CGPoint(x: (compareArcDistance-compareArrowLength)*cos((offsetAngle+sizeOfCompareAngle-compareArrowAngle)*(.pi)/180), y: -(compareArcDistance-compareArrowLength)*sin((offsetAngle+sizeOfCompareAngle-compareArrowAngle)*(.pi)/180)))
            compareAngleArrow.addLine(to: center+CGPoint(x: (compareArcDistance+compareArrowLength)*cos((offsetAngle+sizeOfCompareAngle-compareArrowAngle)*(.pi)/180), y: -(compareArcDistance+compareArrowLength)*sin((offsetAngle+sizeOfCompareAngle-compareArrowAngle)*(.pi)/180)))
            compareAngleArrow.addLine(to: center+CGPoint(x: (compareArcDistance)*cos((offsetAngle+sizeOfCompareAngle)*(.pi)/180), y: -(compareArcDistance)*sin((offsetAngle+sizeOfCompareAngle)*(.pi)/180)))
            compareAngleArrow.closePath()
            compareAngleArrow.setLineJoin(.round)
            compareAngleArrow.setFillColor(colorOfComparableLines.cgColor)
            compareAngleArrow.fillPath()

            compareAngleIsDrawn = false
        }
    }
    
    
    func redrawAngle(offset: CGFloat, size: CGFloat){
        offsetAngle = offset
        sizeOfAngle = size
        touchPoint.center = self.convert(self.center, from: self.superview)+CGPoint(x: lengthOfLine*cos((-offsetAngle-sizeOfAngle)*(.pi)/180), y: lengthOfLine*sin((-offsetAngle-sizeOfAngle)*(.pi)/180))
        self.setNeedsDisplay()
    }
    
    
    var compareAngleIsDrawn = Bool(false)
    func drawCompareAngle(size: CGFloat){
        compareAngleIsDrawn = true
        sizeOfCompareAngle = size
        self.setNeedsDisplay()
    }
    
    
    func disableTouchPoint(){
        touchPoint.alpha = 0
    }
    func enableTouchPoint(){
        touchPoint.alpha = 1
    }

    func disableInteraction(){
        self.isUserInteractionEnabled = false
        touchPoint.alpha = 0.5
    }
    func enableInteration(){
        self.isUserInteractionEnabled = true
        touchPoint.alpha = 1
    }

}
