//
//  EstimateAngleViewController.swift
//  Winkel
//
//  Created by Heiko Etzold on 21.01.18.
//  Copyright © 2018 Heiko Etzold. All rights reserved.
//

import UIKit

class EstimateAngleViewController: AngleViewController, UIPickerViewDelegate, UIPickerViewDataSource  {
    
    var buttonTintColor : UIColor!

    @IBOutlet weak var checkButton: UIButton!

    var currentSizeOfAngle = CGFloat(0)
    var currentOffsetAngle = CGFloat(0)

    @IBOutlet weak var taskLabel: UILabel!

    @IBOutlet weak var scoreLabel: UILabel!

    @IBOutlet weak var estimateAngleView: AngleView!

    @IBOutlet weak var pickerOutlet: UIPickerView!
    var pickerData = ["0°"]
    @IBOutlet weak var downsizeButton: UIButton!
    @IBOutlet weak var enlargeButton: UIButton!

    var listOfInfoLabels : [UILabel] = []
    @IBOutlet weak var exactAngleInfoLabel: UILabel!
    @IBOutlet weak var exactAngleLabel: UILabel!
    @IBOutlet weak var differenceAngleInfoLabel: UILabel!
    @IBOutlet weak var differenceAngleLabel: UILabel!
    @IBOutlet weak var currentPointsLabel: UILabel!
    
    

    override func viewDidLoad() {
        super.viewDidLoad()

        angleView = estimateAngleView

        for i in 1...360{
            pickerData.append("\(i)°")
        }
        pickerOutlet.delegate = self
        pickerOutlet.dataSource = self
        
        
        buttonTintColor = checkButton.tintColor
        
        listOfInfoLabels = [exactAngleInfoLabel, exactAngleLabel, differenceAngleInfoLabel, differenceAngleLabel, currentPointsLabel]

        initTouchPoint()
        
        newTask()
    }
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }


    
    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        return 1
    }
    
    func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        return pickerData.count
    }

    func pickerView(_ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
        return String(pickerData[row])
    }
    
    func pickerView(_ pickerView: UIPickerView, attributedTitleForRow row: Int, forComponent component: Int) -> NSAttributedString? {
        return NSAttributedString(string: pickerData[row], attributes: [NSAttributedString.Key.foregroundColor: buttonTintColor!])
    }
    
    
    //check answer
    @IBAction func pressCheckButton(_ sender: UIButton) {
        
        //only if button really allows to checking (and not to create a new task)
        if(stateOfCheckButton == "check"){
            
            var selectedSizeOfAngle = CGFloat()
            var differenceAngleSize = CGFloat(100)
            
            //identify
            if(currentStateOfTask == 0){
                
                //disable picker
                pickerOutlet.isUserInteractionEnabled = false
                pickerOutlet.alpha = 0.5
                enlargeButton.isEnabled = false
                downsizeButton.isEnabled = false

                //calculate size and change labels
                selectedSizeOfAngle = CGFloat(pickerOutlet.selectedRow(inComponent: 0))
                exactAngleInfoLabel.text = NSLocalizedString("estimate:response:correct", comment: "estimate:response:correct")
                exactAngleInfoLabel.textColor = labelColor
                exactAngleLabel.text = "\(Int(currentSizeOfAngle))°"
                exactAngleLabel.textColor = labelColor
                differenceAngleSize = abs(currentSizeOfAngle-selectedSizeOfAngle)
                differenceAngleLabel.text = "\(Int(differenceAngleSize))°"
                
                //show estimates and correct angles
                estimateAngleView.redrawAngle(offset: currentOffsetAngle, size: currentSizeOfAngle)
                estimateAngleView.drawCompareAngle(size: selectedSizeOfAngle)
            }
                
            //adjust
            else{
                //disable interaction
                estimateAngleView.disableInteraction()

                //calculate size and change labels
                selectedSizeOfAngle = estimateAngleView.sizeOfAngle

                exactAngleInfoLabel.textColor = buttonTintColor
                exactAngleInfoLabel.text = NSLocalizedString("estimate:response:estimation", comment: "estimate:response:estimation")
                exactAngleLabel.textColor = buttonTintColor
                exactAngleLabel.text = "\(Int(selectedSizeOfAngle))°"
                
                differenceAngleSize = abs(currentSizeOfAngle-selectedSizeOfAngle)
                differenceAngleLabel.text = "\(Int(differenceAngleSize))°"

                //show estimates and correct angles
                angleView.colorOfComparableLines = labelColor
                estimateAngleView.redrawAngle(offset: currentOffsetAngle, size: selectedSizeOfAngle)
                estimateAngleView.drawCompareAngle(size: currentSizeOfAngle)
            }
            
            //show all labels
            for label in self.listOfInfoLabels{
                label.alpha = 1
            }

            //shot current points
            currentPointsLabel.text = "\(NSLocalizedString("estimate:response:points", comment: "estimate:response:points")) \(Int(pointsByDifference(difference: differenceAngleSize))) "
            if(pointsByDifference(difference: differenceAngleSize) != 1){
                currentPointsLabel.text = currentPointsLabel.text!+NSLocalizedString("points", comment: "points")+"."
            }
            else{
                currentPointsLabel.text = currentPointsLabel.text!+NSLocalizedString("point", comment: "point")+"."
            }
            
            //show full point, don't do negative points, but show these in a good language
            score += pointsByDifference(difference: differenceAngleSize)
            if(score < 0){
                score = 0
            }
            scoreLabel.text = NSNumber.init(value: score).description(withLocale: NSLocale.current)+" "
            if(score == 1){
                scoreLabel.text = scoreLabel.text!+NSLocalizedString("point", comment: "point")
            }
            else{
                scoreLabel.text = scoreLabel.text!+NSLocalizedString("points", comment: "points")
            }
            
            //change button for a new task
            checkButton.setTitle(NSLocalizedString("next task", comment: "next task"), for: .normal)
            stateOfCheckButton = "newTask"
        }
            
        //if button wants to make a new task
        else{
            newTask()
        }
    }
    
    
    //create a new task
    override func newTask(){
                
        //renew check button
        stateOfCheckButton = "check"
        self.checkButton.setTitle(NSLocalizedString("check", comment: "check"), for: .normal)
        
        //disable all labels
        for label in listOfInfoLabels{
            label.alpha = 0
        }

        
        //create new angle size
        currentSizeOfAngle = CGFloat(Int.random(in: 0...360))
        var drawnSizeOfAngle = currentSizeOfAngle

        
        //identify
        if(currentStateOfTask == 0){
            //enable or disable some other elements
            pickerOutlet.isUserInteractionEnabled = true
            pickerOutlet.alpha = 1
            enlargeButton.alpha = 1
            enlargeButton.isEnabled = true
            downsizeButton.alpha = 1
            downsizeButton.isEnabled = true
            angleView.colorOfLines = labelColor
            angleView.colorOfComparableLines = buttonTintColor
            estimateAngleView.disableTouchPoint()

            // create task
            taskLabel.text = NSLocalizedString("estimate:task:identify", comment: "estimate:task:identify")
        }
            
        //adjust
        else{
            //enable or disable some other elements
            pickerOutlet.alpha = 0
            enlargeButton.alpha = 0
            downsizeButton.alpha = 0
            angleView.colorOfLines = buttonTintColor
            estimateAngleView.enableTouchPoint()
            estimateAngleView.enableInteration()
            
            taskLabel.text = "\(NSLocalizedString("estimate:task:adjust:firstPart", comment: "estimate:task:adjust:firstPart"))\(Int(currentSizeOfAngle))\(NSLocalizedString("estimate:task:adjust:secondPart", comment: "estimate:task:adjust:secondPart"))"
            while(abs(drawnSizeOfAngle-currentSizeOfAngle) < 10){
                drawnSizeOfAngle = CGFloat(Int.random(in: 0...360))
            }
        }
        
        //draw new angle
        currentOffsetAngle = CGFloat([0,90,180,270].randomElement()!)
        estimateAngleView.redrawAngle(offset: currentOffsetAngle, size: drawnSizeOfAngle)
    }
    
    
    //calculate points by differene of angles
    func pointsByDifference(difference: CGFloat) -> Double{
        switch difference {
        case 0..<5:
            return 3
        case 5..<10:
            return 2
        case 10..<15:
            return 1
        default:
            return 0
        }
    }
    
    
    //downsize and enlarge angle by button next to picker
    @IBAction func downsizePicker(_ sender: UIButton) {
        if(pickerOutlet.selectedRow(inComponent: 0)<45){
            pickerOutlet.selectRow(0, inComponent: 0, animated: true)
        }
        else{
            pickerOutlet.selectRow(pickerOutlet.selectedRow(inComponent: 0)-45, inComponent: 0, animated: true)
        }
    }
    @IBAction func enlargePicker(_ sender: UIButton) {
         if(pickerOutlet.selectedRow(inComponent: 0)>315){
             pickerOutlet.selectRow(360, inComponent: 0, animated: true)
         }
         else{
             pickerOutlet.selectRow(pickerOutlet.selectedRow(inComponent: 0)+45, inComponent: 0, animated: true)
         }
     }
}

