//
//  ExploreAngleView.swift
//  Winkel
//
//  Created by Heiko Etzold on 22.01.18.
//  Copyright © 2018 Heiko Etzold. All rights reserved.
//

import UIKit

class ExploreAngleView: UIView {

    var angleA = CGFloat(0)
    var angleB = CGFloat(0)
    var angleG = CGFloat(0)

    var isLockButtonLocked = Bool(false)
    var lockButton = UIButton(type: .system)
    
    var parentViewController : ExploreAngleViewController!
    init(frame: CGRect, parentViewController: ExploreAngleViewController) {
        super.init(frame: frame)
        self.parentViewController = parentViewController
        self.backgroundColor = .clear
        
        lockButton.imageView?.contentMode = .scaleAspectFit
        if #available(iOS 13.0, *) {
            lockButton.setImage(UIImage(systemName: "lock.open.fill"), for: .normal)
        } else {
            lockButton.imageEdgeInsets.top = 15
            lockButton.imageEdgeInsets.bottom = 15
            lockButton.setImage(UIImage(named: "lock.open.fill"), for: .normal)
        }
        if #available(iOS 13.4, *) {
            lockButton.isPointerInteractionEnabled = true
        }
        
        lockButton.tintColor = tintColor
        lockButton.frame.size = CGSize(width: 50, height: 50)
        lockButton.layer.cornerRadius = 25
        lockButton.backgroundColor = orthogonalLineColor.withAlphaComponent(0.5)
        lockButton.addTarget(self, action: #selector(changeLockButton), for: .touchUpInside)
        lockButton.alpha = 0
        self.addSubview(lockButton)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    var orthogonalPoint = CGPoint.zero
    
    
    //change lock button, when pressed
    @objc func changeLockButton(sender: UIButton){
        if(isLockButtonLocked){
            if #available(iOS 13.0, *) {
                lockButton.setImage(UIImage(systemName: "lock.open.fill"), for: .normal)
            } else {
                lockButton.setImage(UIImage(named: "lock.open.fill"), for: .normal)
            }

        }
        else{
            //fix lines to be really parallel
            if #available(iOS 13.0, *) {
                lockButton.setImage(UIImage(systemName: "lock.fill"), for: .normal)
            } else {
                lockButton.setImage(UIImage(named: "lock.fill"), for: .normal)
            }
                        
            let pointInfinityOrthogonalToA = homogCross(firstVector: [parentViewController.lineA[1],-parentViewController.lineA[0],0], secondVector: [0,0,1])
            let lineOrthogonalToAThroughFirstPoint = homogCross(firstVector: [parentViewController.listOfTouchPoints[0].center.x,parentViewController.listOfTouchPoints[0].center.y,1], secondVector: pointInfinityOrthogonalToA)
            let pointOnB = homogCross(firstVector: parentViewController.lineB, secondVector: lineOrthogonalToAThroughFirstPoint)
            let pointInfinityInDirectionB = [parentViewController.lineB[1],-parentViewController.lineB[0],0]
            let lineParallelToBThroughtFirstPoint = homogCross(firstVector: [parentViewController.listOfTouchPoints[1].center.x,parentViewController.listOfTouchPoints[1].center.y,1], secondVector: pointInfinityInDirectionB)
            let pointInfinityOrthogonalToB = homogCross(firstVector: [parentViewController.lineB[1],-parentViewController.lineB[0],0], secondVector: [0,0,1])
            let lineOrthogonalToBThroughPointOnB = homogCross(firstVector: pointOnB, secondVector: pointInfinityOrthogonalToB)
            parentViewController.listOfTouchPoints[0].center = pointByLines(firstLine: lineOrthogonalToBThroughPointOnB, secondLine: lineParallelToBThroughtFirstPoint)
        }
        parentViewController.updateQuarters()
        parentViewController.exploreDrawnAngleView.setNeedsDisplay()
        isLockButtonLocked = !isLockButtonLocked

    }
    
    override func draw(_ rect: CGRect) {
    
        //resize angles
        if(angleG<0){
            angleG+=(.pi)
        }
        while(angleA-angleG < 0){
            angleA+=(.pi)
        }
        while(angleA-angleG > .pi){
            angleA-=(.pi)
        }

        while(angleB-angleG < 0){
            angleB+=(.pi)
        }
        while(angleB-angleG > .pi){
            angleB-=(.pi)
        }


        
        //draw orthogonal line
        if(lockButton.alpha == 1){
            let orthogonal = UIGraphicsGetCurrentContext()!
            
            if(isLockButtonLocked){
                let angle = atan2(parentViewController.listOfTouchPoints[0].center.y-orthogonalPoint.y, parentViewController.listOfTouchPoints[0].center.x-orthogonalPoint.x)
                
                let orthogonalPoints = UIGraphicsGetCurrentContext()!
                orthogonalPoints.setFillColor(orthogonalLineColor.cgColor)

                orthogonalPoints.fillEllipse(in: CGRect(origin: orthogonalPoint+15*CGPoint(x: cos(angle+(.pi/4)), y: sin(angle+(.pi/4)))-CGPoint(x: 2, y: 2), size: CGSize(width: 4, height: 4)))
                orthogonalPoints.fillEllipse(in: CGRect(origin: parentViewController.listOfTouchPoints[0].center+16*CGPoint(x: cos(angle+(.pi)-(.pi/4)), y: sin(angle+(.pi)-(.pi/4)))-CGPoint(x: 2, y: 2), size: CGSize(width: 4, height: 4)))

                orthogonal.move(to: orthogonalPoint+24*CGPoint(x: cos(angle), y: sin(angle)))
                orthogonal.addArc(center: orthogonalPoint, radius: 24, startAngle: angle, endAngle: angle+(.pi/2), clockwise: false)
                
                orthogonal.move(to: parentViewController.listOfTouchPoints[0].center+24*CGPoint(x: cos(angle+(.pi)), y: sin(angle+(.pi))))
                orthogonal.addArc(center: parentViewController.listOfTouchPoints[0].center, radius: 24, startAngle: angle+(.pi), endAngle: angle+(.pi)-(.pi/2), clockwise: true)

            }

            orthogonal.setStrokeColor(orthogonalLineColor.cgColor)
            
            orthogonal.move(to: parentViewController.listOfTouchPoints[0].center)
            orthogonal.addLine(to: orthogonalPoint)

            orthogonal.setLineWidth(2)
            orthogonal.strokePath()
        }
        
        //draw lines
        let lines = UIGraphicsGetCurrentContext()!

        lines.move(to: parentViewController.listOfTouchPoints[0].center-CGPoint(x: 2000*cos(angleA), y: 2000*sin(angleA)))
        lines.addLine(to: parentViewController.listOfTouchPoints[1].center+CGPoint(x: 2000*cos(angleA), y: 2000*sin(angleA)))

        lines.move(to: parentViewController.listOfTouchPoints[2].center-CGPoint(x: 2000*cos(angleB), y: 2000*sin(angleB)))
        lines.addLine(to: parentViewController.listOfTouchPoints[3].center+CGPoint(x: 2000*cos(angleB), y: 2000*sin(angleB)))

        lines.move(to: parentViewController.listOfTouchPoints[4].center-CGPoint(x: 2000*cos(angleG), y: 2000*sin(angleG)))
        lines.addLine(to: parentViewController.listOfTouchPoints[5].center+CGPoint(x: 2000*cos(angleG), y: 2000*sin(angleG)))

        lines.setLineWidth(3)
        lines.setStrokeColor(labelColor.cgColor)
        lines.strokePath()
        
    }
    
    
    //only detect touch in view, if lockButton is visible and touched
    override func point(inside point: CGPoint, with event: UIEvent?) -> Bool {
        return (lockButton.point(inside: lockButton.convert(point, from: self), with: event) && lockButton.alpha == 1)
    }

}
