//
//  ExploreAngleViewController.swift
//  Winkel
//
//  Created by Heiko Etzold on 22.01.18.
//  Copyright © 2018 Heiko Etzold. All rights reserved.
//

import UIKit

class ExploreAngleViewController: UIViewController {
    
    var listOfQuarters : [SpecialAnglePart] = []
    var listOfTouchPoints : [TouchPoint] = []
    
    @IBOutlet var exploreAngleView: UIView!

    var exploreDrawnAngleView : ExploreAngleView!
    override func viewDidLoad() {
        super.viewDidLoad()
        
        exploreDrawnAngleView = ExploreAngleView(frame: exploreAngleView.bounds, parentViewController: self)

        //create touchpoints
        var initPositions : [[CGFloat]] {
            if(UIDevice.current.userInterfaceIdiom == .phone){
                return [[0.1,0.4],[0.9,0.2],[0.1,0.8],[0.9,0.7],[0.1,0.1],[0.75,0.85]]
            }
            return [[0.2,0.2],[0.4,0.8],[0.8,0.1],[0.6,0.75],[0.15,0.55],[0.9,0.3]]
        }

        for pos in initPositions{
            let touchPoint = TouchPoint(position: CGPoint(x: pos[0]*exploreDrawnAngleView.frame.width, y: pos[1]*exploreDrawnAngleView.frame.height), connectedViewController: self, small: true)
            touchPoint.index = initPositions.firstIndex(of: pos)!
            listOfTouchPoints += [touchPoint]
            view.addSubview(touchPoint)
        }
        
        //create quarters
        for i in 0...7{
            let quarter = SpecialAnglePart(index: i, parentViewController: self)
            quarter.isSizeInformatable = true
            exploreAngleView.addSubview(quarter)
            exploreAngleView.sendSubviewToBack(quarter)
            listOfQuarters += [quarter]
        }

        //make everything new
        exploreAngleView.addSubview(exploreDrawnAngleView)
        
        exploreDrawnAngleView.translatesAutoresizingMaskIntoConstraints = false
        exploreDrawnAngleView.widthAnchor.constraint(equalTo: exploreAngleView.widthAnchor).isActive = true
        exploreDrawnAngleView.heightAnchor.constraint(equalTo: exploreAngleView.heightAnchor).isActive = true
        exploreDrawnAngleView.centerXAnchor.constraint(equalTo: exploreAngleView.centerXAnchor).isActive = true
        exploreDrawnAngleView.centerYAnchor.constraint(equalTo: exploreAngleView.centerYAnchor).isActive = true

        updateQuarters()
        exploreDrawnAngleView.setNeedsDisplay()
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    
    var lineA : [CGFloat] = [0,0,1]
    var lineB : [CGFloat]  = [0,0,1]
    var lineG : [CGFloat]  = [0,0,1]

    func updateQuarters(){
        //calc lines
        lineA = lineByPoints(firstPoint: listOfTouchPoints[0].center, secondPoint: listOfTouchPoints[1].center)
        lineB = lineByPoints(firstPoint: listOfTouchPoints[2].center, secondPoint: listOfTouchPoints[3].center)
        lineG = lineByPoints(firstPoint: listOfTouchPoints[4].center, secondPoint: listOfTouchPoints[5].center)
        
        //calc angles
        let G1 = pointByLines(firstLine: lineA, secondLine: lineG)
        let G2 = pointByLines(firstLine: lineB, secondLine: lineG)
        exploreDrawnAngleView.angleA = atan2(listOfTouchPoints[1].center.y-listOfTouchPoints[0].center.y,listOfTouchPoints[1].center.x-listOfTouchPoints[0].center.x)
        exploreDrawnAngleView.angleB = atan2(listOfTouchPoints[3].center.y-listOfTouchPoints[2].center.y,listOfTouchPoints[3].center.x-listOfTouchPoints[2].center.x)
        exploreDrawnAngleView.angleG = atan2(listOfTouchPoints[5].center.y-listOfTouchPoints[4].center.y,listOfTouchPoints[5].center.x-listOfTouchPoints[4].center.x)

        //resize angles (angles are very compicated!)
        while(exploreDrawnAngleView.angleA-exploreDrawnAngleView.angleG < 0){
            exploreDrawnAngleView.angleA+=(.pi)
        }
        while(exploreDrawnAngleView.angleA-exploreDrawnAngleView.angleG > .pi){
            exploreDrawnAngleView.angleA-=(.pi)
        }

        while(exploreDrawnAngleView.angleB-exploreDrawnAngleView.angleG < 0){
            exploreDrawnAngleView.angleB+=(.pi)
        }
        while(exploreDrawnAngleView.angleB-exploreDrawnAngleView.angleG > .pi){
            exploreDrawnAngleView.angleB-=(.pi)
        }
        
        //draw all the quarters
        for quarter in listOfQuarters{
            
            var refPoint = G1
            var refAngle = exploreDrawnAngleView.angleA
            if(quarter.index>=4){
                refPoint = G2
                refAngle = exploreDrawnAngleView.angleB
            }
            
            var startAngle = exploreDrawnAngleView.angleG
            if(quarter.index%2==1){
                startAngle = refAngle
            }
            if(2...3 ~= quarter.index%4){
                startAngle += .pi
            }
            
            var endAngle = exploreDrawnAngleView.angleG
            if(quarter.index%2==0){
                endAngle = refAngle
            }
            if(1...2 ~= quarter.index%4){
                endAngle += .pi
            }
            
            quarter.renewPosition(center: refPoint, startAngle: startAngle, endAngle: endAngle)
        }
    }

    
    

    //update view, when Mac-window is resized
    override func viewWillTransition(to size: CGSize, with coordinator: UIViewControllerTransitionCoordinator) {
        if(listOfTouchPoints.count != 0){
            exploreDrawnAngleView.setNeedsDisplay()
        }
    }
    
}
