//
//  Globals.swift
//  Winkel
//
//  Created by Heiko Etzold on 06.04.20.
//  Copyright © 2020 Heiko Etzold. All rights reserved.
//

import Foundation
import UIKit

extension UILabel {
    @IBInspectable var localizableText: String? {
        get { return text }
        set(value) { text = NSLocalizedString(value!, comment: "") }
    }
}

extension UIButton {
    @IBInspectable var localizableText: String? {
        get { return titleLabel?.text }
        set(value) { titleLabel?.text = NSLocalizedString(value!, comment: "") }
    }
}

func distance(firstPoint: CGPoint, secondPoint: CGPoint) -> CGFloat{
    return sqrt((firstPoint.x-secondPoint.x)*(firstPoint.x-secondPoint.x)+(firstPoint.y-secondPoint.y)*(firstPoint.y-secondPoint.y))
}

//functions to calculate with CGPoints
extension CGPoint {
    static func +(left: CGPoint, right: CGPoint) -> CGPoint {
        return CGPoint(x: left.x+right.x, y: left.y+right.y)
    }
    
    static func +=( left: inout CGPoint, right: CGPoint){
        left = left+right
    }
    
    static func -(left: CGPoint, right: CGPoint) -> CGPoint {
        return CGPoint(x: left.x-right.x, y: left.y-right.y)
    }
    
    static func -=( left: inout CGPoint, right: CGPoint){
        left = left-right
    }
    
    static func *(left: CGFloat, right: CGPoint) -> CGPoint {
        return CGPoint(x: left*right.x, y: left*right.y)
    }
}

//some functions to calculate with homogenious coordinates
func lineByPoints(firstPoint: CGPoint, secondPoint: CGPoint) -> [CGFloat] {
    return [firstPoint.y-secondPoint.y,secondPoint.x-firstPoint.x,firstPoint.x*secondPoint.y-firstPoint.y*secondPoint.x]
}
func pointByLines(firstLine: [CGFloat], secondLine: [CGFloat]) -> CGPoint{
    let x = firstLine[1]*secondLine[2]-firstLine[2]*secondLine[1]
    let y = firstLine[2]*secondLine[0]-firstLine[0]*secondLine[2]
    let z = firstLine[0]*secondLine[1]-firstLine[1]*secondLine[0]

    if(z != 0){
        return CGPoint(x: x/z, y: y/z)
    }
    return CGPoint.zero
}
func homogCross(firstVector: [CGFloat], secondVector: [CGFloat]) -> [CGFloat]{
    let x = firstVector[1]*secondVector[2]-firstVector[2]*secondVector[1]
    let y = firstVector[2]*secondVector[0]-firstVector[0]*secondVector[2]
    let z = firstVector[0]*secondVector[1]-firstVector[1]*secondVector[0]

    return[x,y,z]
}



var labelColor : UIColor{
    if #available(iOS 13.0, *) {
        return UIColor.label
    }
    return UIColor.black
}

var grayColor : UIColor{
    if #available(iOS 13.0, *) {
        return UIColor.opaqueSeparator
    }
    return UIColor.systemGray.withAlphaComponent(0.5)
}


var orthogonalLineColor : UIColor{
    if #available(iOS 13.0, *) {
        return UIColor.systemGray3
    }
    return UIColor.lightGray
}



