//
//  KindOfAngleViewController.swift
//  Winkel
//
//  Created by Heiko Etzold on 21.01.18.
//  Copyright © 2018 Heiko Etzold. All rights reserved.
//

import UIKit

class KindOfAngleViewController: AngleViewController {

    var listOfButtons : [UIButton] = []
    @IBOutlet weak var button0: UIButton!
    @IBOutlet weak var button1: UIButton!
    @IBOutlet weak var button2: UIButton!
    @IBOutlet weak var button3: UIButton!
    @IBOutlet weak var button4: UIButton!
    @IBOutlet weak var button5: UIButton!
    @IBOutlet weak var button6: UIButton!
    var buttonTintColor : UIColor!

    @IBOutlet weak var checkButton: UIButton!

    var currentKindOfAngle = Int(2)
    var selectedKindOfAngle = Int(-1)

    var currentSizeOfAngle = CGFloat(0)
    var currentOffsetAngle = CGFloat(0)

    @IBOutlet weak var taskLabel: UILabel!
    let dictionaryOfAngleTasks : [String] = [NSLocalizedString("kind:task:adjust:zero", comment: "kind:task:adjust:zero"),
                                         NSLocalizedString("kind:task:adjust:acute", comment: "kind:task:adjust:acute"),
                                         NSLocalizedString("kind:task:adjust:right", comment: "kind:task:adjust:right"),
                                         NSLocalizedString("kind:task:adjust:obtuse", comment: "kind:task:adjust:obtuse"),
                                         NSLocalizedString("kind:task:adjust:straight", comment: "kind:task:adjust:straight"),
                                         NSLocalizedString("kind:task:adjust:reflex", comment: "kind:task:adjust:reflex"),
                                         NSLocalizedString("kind:task:adjust:round", comment: "kind:task:adjust:round")
    ]


    @IBOutlet weak var scoreLabel: UILabel!
    
    @IBOutlet weak var kindOfAngleView: AngleView!


    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        angleView = kindOfAngleView
        
        listOfButtons = [button0, button1, button2, button3, button4, button5, button6]
        buttonTintColor = button0.tintColor
        
        initTouchPoint()
        
        newTask()
    }
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    
    //press kind-of-angle button
    @IBAction func pressButton(_ sender: UIButton) {

        selectedKindOfAngle = Int(-1)

        //identify pressed kind of angle
        for button in listOfButtons{
            if(sender == button){
                if(sender.isSelected != true){
                    sender.isSelected = true
                    selectedKindOfAngle = button.tag
                }
                else{
                    sender.isSelected = false
                }
            }
            else{
                button.isSelected = false
            }
        }

        //make the button nice again
        if(selectedKindOfAngle == -1){
            checkButton.isEnabled = false
        }
        else{
            checkButton.isEnabled = true
        }
    }
    
    
    //check answer
    @IBAction func pressCheckButton(_ sender: UIButton) {
        
        //only if button really allows to checking (and not to create a new task)
        if(stateOfCheckButton == "check"){
            
            //identify
            if(currentStateOfTask == 0){
                
                //if answer is correct
                if(currentKindOfAngle == selectedKindOfAngle){
                    
                    //disable all buttons and show correct with a nice green animation
                    UIView.animate(withDuration: 0.15, animations: {
                        self.listOfButtons[self.selectedKindOfAngle].tintColor = UIColor.systemGreen
                    })
                    for button in self.listOfButtons{
                        button.isEnabled = false
                    }
                    listOfButtons[selectedKindOfAngle].isEnabled = true
                    listOfButtons[selectedKindOfAngle].isUserInteractionEnabled = false
                    
                    //change button for a new task
                    checkButton.setTitle(NSLocalizedString("next task", comment: "next task"), for: .normal)
                    stateOfCheckButton = "newTask"
                    
                    //give me one point!
                    score += 1
                }
                    
                //if answer ist wrong :(
                else{
                    //remember color of button and make it read (quickly and shortly)
                    let savedColor = self.listOfButtons[self.selectedKindOfAngle].backgroundColor
                    UIView.animate(withDuration: 0.15, animations: {
                        self.listOfButtons[self.selectedKindOfAngle].backgroundColor = UIColor.systemRed
                    }, completion: { (finished: Bool) in
                        UIView.animate(withDuration: 0.15, animations: {
                            self.listOfButtons[self.selectedKindOfAngle].backgroundColor = savedColor
                        })
                    })
                    
                    //subtract a half poin
                    score -= 0.5
                }
            }
                
            //adjust
            else{
                //if answer ist correct
                if(isAngleInRange(sizeOfAngle: kindOfAngleView.sizeOfAngle, kindOfAngle: currentKindOfAngle)){
            
                    //change button for a new task
                    kindOfAngleView.disableInteraction()
                    
                    //show a green response (animated)
                    UIView.animate(withDuration: 0.15, animations: {
                        self.checkButton.tintColor = .systemGreen
                        self.checkButton.isSelected = true
                    }, completion: { (finished: Bool) in
                        self.checkButton.setTitle(NSLocalizedString("next task", comment: "next task"), for: .normal)
                        self.stateOfCheckButton = "newTask"
                    })
                    
                    //one more point
                    score += 1
                }
                    
                //if answer is wrong
                else{
                    
                    //show a red response (also animated)
                    UIView.animate(withDuration: 0.15, animations: {
                        self.checkButton.tintColor = .systemRed
                        self.checkButton.isSelected = true
                    }, completion: { (finished: Bool) in
                        UIView.animate(withDuration: 0.15, animations: {
                            self.checkButton.tintColor = self.buttonTintColor
                            self.checkButton.isSelected = false
                        })
                    })
                    
                    //baaaaad
                    score -= 0.5
                }
            }

            //don't do negative points, but show these in a good language
            if(score < 0){
                score = 0
            }
            scoreLabel.text = NSNumber.init(value: score).description(withLocale: NSLocale.current)+" "
            if(score == 1){
                scoreLabel.text = scoreLabel.text!+NSLocalizedString("point", comment: "point")
            }
            else{
                scoreLabel.text = scoreLabel.text!+NSLocalizedString("points", comment: "points")
            }
        }
            
        //if button wants to make a new task
        else{
            newTask()
        }
    }
    
    
    //create a new task
    override func newTask(){
                
        //renew check button
        stateOfCheckButton = "check"
        checkButton.isSelected = false
        checkButton.tintColor = buttonTintColor
        self.checkButton.setTitle(NSLocalizedString("check", comment: "check"), for: .normal)
        
        //renew kind-of-angle buttons
        for button in listOfButtons{
            button.tintColor = buttonTintColor
            button.isSelected = false
            button.isEnabled = true
            button.isUserInteractionEnabled = true
        }

        
        //create random kind of angle
        currentKindOfAngle = Int.random(in: 0 ... 6)
        
        
        //if angle has to be identified
        if(currentStateOfTask == 0){
            
            for button in listOfButtons{
                button.alpha = 1
            }
            checkButton.isEnabled = false
            kindOfAngleView.disableTouchPoint()
            angleView.colorOfLines = labelColor
            
            checkButton.isEnabled = false  // diable check button
            taskLabel.text = NSLocalizedString("kind:task:identify", comment: "kind:task:identify")  // create task
            currentSizeOfAngle = randomAngleByKindOfAngle(kindOfAngle: currentKindOfAngle)  // create random angle
        }
        //if angle has to be adjusted
        else{
            for button in listOfButtons{
                button.alpha = 0
            }
            angleView.colorOfLines = buttonTintColor
            checkButton.isEnabled = true
            kindOfAngleView.enableInteration()  // allow interaction
            taskLabel.text = dictionaryOfAngleTasks[currentKindOfAngle]  // create task
            var otherKindOfAngle = currentKindOfAngle // create another kind of angle
            while(otherKindOfAngle == currentKindOfAngle){
                otherKindOfAngle = Int.random(in: 0 ... 6)
            }
            currentSizeOfAngle = randomAngleByKindOfAngle(kindOfAngle: otherKindOfAngle)
        }
        
        //draw angle
        currentOffsetAngle = CGFloat.random(in: 0 ..< 360)
        kindOfAngleView.redrawAngle(offset: currentOffsetAngle, size: currentSizeOfAngle)
    }
    
    
    //check, if size matches given kind of angle
    func isAngleInRange (sizeOfAngle: CGFloat, kindOfAngle: Int) -> Bool{
        switch kindOfAngle {
        case 1:
            return 0 < sizeOfAngle && sizeOfAngle < 90
        case 2:
            return 85 < sizeOfAngle && sizeOfAngle < 95
        case 3:
            return 90 < sizeOfAngle && sizeOfAngle < 180
        case 4:
            return 175 < sizeOfAngle && sizeOfAngle < 185
        case 5:
            return 180 < sizeOfAngle && sizeOfAngle < 360
        case 6:
            return sizeOfAngle == 360
        default:
            return sizeOfAngle == 0
        }
    }
    
    
    //create size of angle by kind of angle
    func randomAngleByKindOfAngle(kindOfAngle: Int) -> CGFloat {
        switch kindOfAngle {
        case 1:
            return CGFloat.random(in: 5 ... 85)
        case 2:
            return 90
        case 3:
            return CGFloat.random(in: 95 ... 175)
        case 4:
            return 180
        case 5:
            return CGFloat.random(in: 185 ... 355)
        case 6:
            return 360
        default:
            return 0
        }
    }
}
