//
//  SpecialAnglePart.swift
//  Winkel
//
//  Created by Heiko Etzold on 29.03.20.
//  Copyright © 2020 Heiko Etzold. All rights reserved.
//

import UIKit

class SpecialAnglePart: UIView, UIPointerInteractionDelegate {
    
    var isSelected = Bool(false)
    var isSizeInformatable = Bool(false)
    var index = Int(-1)
    var ownSize = CGFloat(0)
    var parentViewController = UIViewController()
    var isGiven = Bool(false)

    //label for angle size
    var label = UILabel()
    
    init(index: Int, parentViewController: UIViewController) {
        super.init(frame:CGRect(x: 0, y: 0, width: 150, height: 150))
        
        self.parentViewController = parentViewController
        self.index = index
        
        let tapGesture = UITapGestureRecognizer(target: self, action: #selector(tapAngle))
        self.addGestureRecognizer(tapGesture)

        label.alpha = 0
        self.addSubview(label)
        
        //set pointer (for trackpad/mouse)
        if #available(iOS 13.4, *) {
            let pointerInteraction = UIPointerInteraction(delegate: self)
            self.addInteraction(pointerInteraction)
        }
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    
    //change pointer to circle, when quarter is hovered
    @available(iOS 13.4, *)
    func pointerInteraction(_ interaction: UIPointerInteraction, styleFor region: UIPointerRegion) -> UIPointerStyle? {
        var pointerStyle: UIPointerStyle?
        
        if interaction.view != nil {
            let shape = UIBezierPath(ovalIn: CGRect(origin: CGPoint(x: -10, y: -10), size: CGSize(width: 20, height: 20)))
            shape.append(UIBezierPath(ovalIn: CGRect(origin: CGPoint(x: -8, y: -8), size: CGSize(width: 16, height: 16))))
            shape.usesEvenOddFillRule = true
            pointerStyle = UIPointerStyle(shape: UIPointerShape.path(shape))
        }
        return pointerStyle
    }
    
    
    //add blurr view, when pointer is entered
    @available(iOS 13.4, *)
    func pointerInteraction(_ interaction: UIPointerInteraction, willEnter region: UIPointerRegion, animator: UIPointerInteractionAnimating) {
        if interaction.view != nil {
            let blurEffect = UIBlurEffect(style: .systemThinMaterial)
            let blurView = UIVisualEffectView(effect: blurEffect)
            blurView.frame = self.bounds
            self.insertSubview(blurView, at: 0)
        }
    }
    
    //remove blurr view, when pointer is exited
    @available(iOS 13.4, *)
    func pointerInteraction(_ interaction: UIPointerInteraction, willExit region: UIPointerRegion, animator: UIPointerInteractionAnimating) {
        if interaction.view != nil {
            self.subviews.first?.removeFromSuperview()
            
        }
    }
    

    //when angle quarter ist tapped, change color (if allowed) and show size (if is estimating)
    @objc func tapAngle(sender: UITapGestureRecognizer){
        isSelected = !isSelected
        redraw()
        
        if let parent = parentViewController as? SpecialAngleViewController{
            parent.enableButtonsByQuarters()
        }
        if(isSizeInformatable && isSelected){
            label.alpha = 1
        }
        else{
            label.alpha = 0
        }
    }
    
    //only recognize taps inside quarter
    var maskPash = UIBezierPath()
    override func point(inside point: CGPoint, with event: UIEvent?) -> Bool {
        maskPash.contains(point)
    }
    
    //draw quarter new
    func redraw(){
        self.backgroundColor = UIColor.systemGray.withAlphaComponent(0.2)

        if(isSelected){
            if(self.isUserInteractionEnabled){
                self.backgroundColor = tintColor.withAlphaComponent(0.5)
            }
            else{
                self.backgroundColor = tintColor.withAlphaComponent(0.25)
            }
        }
        if(isGiven){
            self.backgroundColor = UIColor.systemGray
        }
                    
    }
    
    //renew quarter, depending on position and angle
    func renewPosition(center: CGPoint, startAngle: CGFloat, endAngle: CGFloat){
        redraw()
        
        self.center = center
        let maskLayer = CAShapeLayer()
        maskPash = UIBezierPath()
        maskPash.move(to: CGPoint(x: self.frame.width/2, y: self.frame.height/2))
        maskPash.addArc(withCenter: CGPoint(x: self.frame.width/2, y: self.frame.height/2), radius: self.frame.width/2, startAngle: startAngle, endAngle: endAngle, clockwise: true)
        maskLayer.path = maskPash.cgPath
        self.layer.mask = maskLayer
        
        ownSize = endAngle-startAngle
        if(ownSize < 0){
            ownSize += 2*(.pi)
        }
        label.text = "≈ \(Int(round(ownSize*180/(.pi))))°"
        label.sizeToFit()
        label.center = CGPoint(x: self.frame.width/2+45*cos(startAngle+ownSize/2), y: self.frame.height/2+45*sin(startAngle+ownSize/2))
    }
    
}
