//
//  SpecialAngleViewController.swift
//  Winkel
//
//  Created by Heiko Etzold on 28.03.20.
//  Copyright © 2020 Heiko Etzold. All rights reserved.
//

import UIKit

class SpecialAngleViewController: AngleViewController, UIPointerInteractionDelegate {
  
    var listOfButtons : [UIButton] = []
    @IBOutlet weak var button0: UIButton!
    @IBOutlet weak var button1: UIButton!
    @IBOutlet weak var button2: UIButton!
    @IBOutlet weak var button3: UIButton!
    @IBOutlet weak var button4: UIButton!
    var buttonTintColor : UIColor!
    
    @IBOutlet weak var checkButton: UIButton!
    
    var currentSpecialAngle = Int(-1)
    var selectedSpecialAngle = Int(-1)

    @IBOutlet weak var taskLabel: UILabel!
    let dictionaryOfAngleTasks : [String] = [NSLocalizedString("special:task:adjust:vertical", comment: "special:task:adjust:vertical"),
                                         NSLocalizedString("special:task:adjust:adjacent", comment: "special:task:adjust:adjacent"),
                                         NSLocalizedString("special:task:adjust:corresponding", comment: "special:task:adjust:corresponding"),
                                         NSLocalizedString("special:task:adjust:alternate", comment: "special:task:adjust:alternate")
    ]

    @IBOutlet weak var responseLabel: UILabel!
    
    @IBOutlet weak var scoreLabel: UILabel!
    var scoreAtSpecialAngle = Double(0)

    @IBOutlet weak var allowNonParallels: UISwitch!
    @IBOutlet weak var allowNonparallelsLabel: UILabel!

    @IBOutlet weak var specialAngleView: UIView!
    let specialAngleDrawnView = SpecialAngleView()

    var listOfQuarters : [SpecialAnglePart] = []
    
    
    
    override func viewDidLoad() {
        super.viewDidLoad()

        specialAngleView.clipsToBounds = true
        
        listOfButtons = [button0, button1, button2, button3, button4]
        buttonTintColor = button0.tintColor
        allowNonParallels.onTintColor = buttonTintColor
        allowNonparallelsLabel.textColor = grayColor

        //create quarters
        for i in 0...7{
            let quarter = SpecialAnglePart(index: i, parentViewController: self)
            specialAngleView.addSubview(quarter)
            listOfQuarters += [quarter]
        }
        
        //create view and add constraints
        specialAngleDrawnView.backgroundColor = .clear
        specialAngleDrawnView.isUserInteractionEnabled = false
        specialAngleView.addSubview(specialAngleDrawnView)
        specialAngleDrawnView.translatesAutoresizingMaskIntoConstraints = false
        specialAngleDrawnView.widthAnchor.constraint(equalTo: specialAngleView.widthAnchor).isActive = true
        specialAngleDrawnView.heightAnchor.constraint(equalTo: specialAngleView.heightAnchor).isActive = true
        specialAngleDrawnView.centerXAnchor.constraint(equalTo: specialAngleView.centerXAnchor).isActive = true
        specialAngleDrawnView.centerYAnchor.constraint(equalTo: specialAngleView.centerYAnchor).isActive = true

        //some changes, when iPhone is used
        if(UIDevice.current.userInterfaceIdiom == .phone){
            currentStateOfTask = 1
        }
        
        //set pointer (trackpad/mouse)
        if #available(iOS 13.4, *) {
            let pointerInteraction = UIPointerInteraction(delegate: self)
            allowNonParallels.addInteraction(pointerInteraction)
        }
        
    }
    
    //change pointer, when UISwitch is hovered
    @available(iOS 13.4, *)
    func pointerInteraction(_ interaction: UIPointerInteraction, styleFor region: UIPointerRegion) -> UIPointerStyle? {
        var pointerStyle: UIPointerStyle? = nil
        
        if let interactionView = interaction.view {
            let targetedPreview = UITargetedPreview(view: interactionView)
            pointerStyle = UIPointerStyle(effect: UIPointerEffect.highlight(targetedPreview), shape: UIPointerShape.roundedRect(interactionView.frame, radius: 20))
        }
        return pointerStyle
    }
    
    
    //create new task, when viewController is opened first time
    var firstTime = Bool(true)
    override func viewDidAppear(_ animated: Bool) {
        if(firstTime){
            newTask()
            firstTime=false
        }
        
    }
    
    
    //change enability of parallel lines
    @IBAction func changeEnabilityOfParallelLines(_ sender: UISwitch) {
        if(sender.isOn){
            allowNonparallelsLabel.textColor = labelColor
        }
        else{
            allowNonparallelsLabel.textColor = grayColor
        }
    }

    
    //press special-angle button
    @IBAction func pressButton(_ sender: UIButton) {
        
        selectedSpecialAngle = Int(-1)

        //identify pressed kind of angle
        for button in listOfButtons{
            if(sender == button){
                if(sender.isSelected != true){
                    sender.isSelected = true
                    selectedSpecialAngle = button.tag
                }
                else{
                    sender.isSelected = false
                }
            }
            else{
                button.isSelected = false
            }
        }

        //make the button nice again
        if(selectedSpecialAngle == -1){
            checkButton.isEnabled = false
        }
        else{
            checkButton.isEnabled = true
        }
    }
    
    
    func showButtonAsRight(buttonIndex: Int){
        //disable all buttons and show correct with a nice green animation
        UIView.animate(withDuration: 0.15, animations: {
            self.listOfButtons[buttonIndex].tintColor = UIColor.systemGreen
        })
        for button in self.listOfButtons{
            button.isEnabled = false
        }
        listOfButtons[buttonIndex].isEnabled = true
        listOfButtons[buttonIndex].isUserInteractionEnabled = false
        
        //change button for a new task
        checkButton.setTitle(NSLocalizedString("next task", comment: "next task"), for: .normal)
        stateOfCheckButton = "newTask"
        
        for quarter in listOfQuarters{
            quarter.isUserInteractionEnabled = false
            quarter.redraw()
        }
        
        //give me one point!
        scoreAtSpecialAngle += 1
    }
    
    
    func showButtonAsWrong(buttonIndex: Int){
        //remember color of button and make it read (quickly and shortly)
        let savedColor = self.listOfButtons[buttonIndex].backgroundColor
        UIView.animate(withDuration: 0.15, animations: {
            self.listOfButtons[buttonIndex].backgroundColor = UIColor.systemRed
        }, completion: { (finished: Bool) in
            UIView.animate(withDuration: 0.15, animations: {
                self.listOfButtons[buttonIndex].backgroundColor = savedColor
            })
        })
        
        //subtract a half poin
        scoreAtSpecialAngle -= 0.5
    }
    
    
    //calculation of the special angles by other angle
    func calcVerticalAngleIndex(angleIndex: Int) -> Int{
        return angleIndex/4*4+(angleIndex+2)%4
    }
    func calcAdjacentAngleIndex(angleIndex: Int) -> Int{
        return angleIndex/4*4+(angleIndex+1)%4
    }
    func calcCorrespondingAngleIndex(angleIndex: Int) -> Int{
        return (angleIndex+4)%8
    }
    func calcAlternateAngleIndex(angleIndex: Int) -> Int{
        return (4+angleIndex/4*4+(angleIndex+2)%4)%8
    }

    //check answer
    @IBAction func pressCheckButton(_ sender: UIButton) {

        //only if button really allows to checking (and not to create a new task)
        if(stateOfCheckButton == "check"){

            switch currentStateOfTask {
            //identify
            case 0:
                //if answer ist correct
                if(currentSpecialAngle == selectedSpecialAngle){
                    showButtonAsRight(buttonIndex: selectedSpecialAngle)
                }
                //if answer ist wrong
                else{
                    showButtonAsWrong(buttonIndex: selectedSpecialAngle)
                }

            //adjust
            case 1:
                var givenQuarterIndex = Int(-1)
                var selectedQuartersIndizes : [Int] = []
                
                //safe selected quarters
                for quarter in listOfQuarters {
                    if(quarter.isSelected){
                        selectedQuartersIndizes += [quarter.index]
                    }
                    if(quarter.isGiven){
                        givenQuarterIndex = quarter.index
                    }
                }
                
                //count correct answers
                var answerIsPartlyCorrect = Bool(false)
                var answerContainsWrongQuarters = Bool(false)
                func checkAnswers(_ condition: Bool){
                    if(condition){
                        answerIsPartlyCorrect = true
                    }
                    else{
                        answerContainsWrongQuarters = true
                    }
                }
                for quarterIndex in selectedQuartersIndizes{
                    switch currentSpecialAngle {
                    case 0:
                        checkAnswers(quarterIndex == calcVerticalAngleIndex(angleIndex: givenQuarterIndex))
                    case 1:
                        checkAnswers(quarterIndex == calcAdjacentAngleIndex(angleIndex: givenQuarterIndex) || givenQuarterIndex == calcAdjacentAngleIndex(angleIndex: quarterIndex))
                    case 2:
                        checkAnswers(quarterIndex == calcCorrespondingAngleIndex(angleIndex: givenQuarterIndex))
                    case 3:
                        checkAnswers(quarterIndex == calcAlternateAngleIndex(angleIndex: givenQuarterIndex))
                    default: break
                    }
                }
                
                //show right response
                responseLabel.text = ""
                //if every answer is wrong
                if(!answerIsPartlyCorrect){
                    showWrongImage()
                }
                else{
                    //if some answer ist wrong
                    if(answerContainsWrongQuarters){
                        showWrongImage()
                        responseLabel.text = NSLocalizedString("special:response:partly", comment: "special:response:partly")
                    }
                    //if no answer is wrong
                    else{
                        //for adjacent angles
                        if(currentSpecialAngle != 1){
                            showRightImage()
                        }
                        //for other angles
                        else{
                            //if something is wrong anymore
                            if(selectedQuartersIndizes.count != 2){
                                showWrongImage()
                                responseLabel.text = NSLocalizedString("special:response:incomplete", comment: "special:response:incomplete")
                            }
                            //if everything is good
                            if(selectedQuartersIndizes.count == 2){
                                showRightImage()
                            }
                        }
                    }
                }
                
            //teamwork
            default:
                //safe selected quarters
                var selectedQuartersIndizes : [Int] = []
                for quarter in listOfQuarters.filter({$0.isSelected}) {
                    if(quarter.isUserInteractionEnabled){
                        selectedQuartersIndizes += [quarter.index]
                    }
                }
                
                //only, if two angles are selected
                if(selectedQuartersIndizes.count == 2){
                    
                    //detect, which special angles are selected
                    let firstAngleIndex = selectedQuartersIndizes[0]
                    let secondAngleIndex = selectedQuartersIndizes[1]
                    var adjustedSpecialAngle : Int {
                        if(firstAngleIndex == calcAdjacentAngleIndex(angleIndex: secondAngleIndex)  || secondAngleIndex == calcAdjacentAngleIndex(angleIndex: firstAngleIndex)){
                            return 1
                        }
                        switch firstAngleIndex {
                        case calcVerticalAngleIndex(angleIndex: secondAngleIndex):
                            return 0
                        case calcCorrespondingAngleIndex(angleIndex: secondAngleIndex):
                            return 2
                        case calcAlternateAngleIndex(angleIndex: secondAngleIndex):
                            return 3
                        default:
                            return 4
                        }
                    }
                    currentSpecialAngle = adjustedSpecialAngle

                    //if answer ist right
                    if(currentSpecialAngle == selectedSpecialAngle){
                        showButtonAsRight(buttonIndex: selectedSpecialAngle)
                    }
                    //if answer ist wrong
                    else{
                        showButtonAsWrong(buttonIndex: selectedSpecialAngle)
                    }
                }
            }
        }
        
        //if button wants to make a new task
        else{
            newTask()
        }
        
        //don't do negative points, but show these in a good language
        if(scoreAtSpecialAngle < 0){
            scoreAtSpecialAngle = 0
        }
        scoreLabel.text = NSNumber.init(value: scoreAtSpecialAngle).description(withLocale: NSLocale.current)+" "
        if(scoreAtSpecialAngle == 1){
            scoreLabel.text = scoreLabel.text!+NSLocalizedString("point", comment: "point")
        }
        else{
            scoreLabel.text = scoreLabel.text!+NSLocalizedString("points", comment: "points")
        }
    }
    
    
    func showRightImage(){
        
        UIView.animate(withDuration: 0.15, animations: {
            self.checkButton.tintColor = .systemGreen
            self.checkButton.isSelected = true
        }, completion: { (finished: Bool) in
            self.checkButton.setTitle(NSLocalizedString("next task", comment: "next task"), for: .normal)
            self.stateOfCheckButton = "newTask"
        })


        //disable quarters
        for quarter in listOfQuarters{
            quarter.isUserInteractionEnabled = false
            quarter.redraw()
        }
        
        
        //one more point
        scoreAtSpecialAngle += 1
    }
    
    
    func showWrongImage(){

        UIView.animate(withDuration: 0.15, animations: {
            self.checkButton.tintColor = UIColor.systemRed
            self.checkButton.isSelected = true
        }, completion: { (finished: Bool) in
            UIView.animate(withDuration: 0.15, animations: {
                self.checkButton.tintColor = self.buttonTintColor
                self.checkButton.isSelected = false
            })
        })

        scoreAtSpecialAngle -= 0.5
    }
    
    
    //create a new task
    override func newTask(){
        
        //renew check button
        stateOfCheckButton = "check"
        checkButton.isSelected = false
        checkButton.tintColor = buttonTintColor
        self.checkButton.setTitle(NSLocalizedString("check", comment: "check"), for: .normal)
        
        //disable response image
        responseLabel.text = ""
        
        //renew special-angle buttons
        for button in listOfButtons{
            button.tintColor = buttonTintColor
            button.isSelected = false
            button.isEnabled = true
            button.isUserInteractionEnabled = true
        }

        //create two points, where angles are living, and the corresponding lines
        let firstAngle = CGFloat.random(in: 0 ..< .pi*2)
        specialAngleDrawnView.pointG1 = createRandomPoint(angle: firstAngle)
        
        let secondAngle = firstAngle + CGFloat.random(in: .pi/2 ... .pi*3/2)
        specialAngleDrawnView.pointG2 = createRandomPoint(angle: secondAngle)
        
        specialAngleDrawnView.pointG = CGFloat(0.5)*(specialAngleDrawnView.pointG1+specialAngleDrawnView.pointG2)
        specialAngleDrawnView.angleG = atan2(specialAngleDrawnView.pointG2.y-specialAngleDrawnView.pointG1.y,specialAngleDrawnView.pointG2.x-specialAngleDrawnView.pointG1.x)
        
        specialAngleDrawnView.angleA = specialAngleDrawnView.angleG + CGFloat.random(in: .pi/6 ... .pi*5/6)
        
        if(allowNonParallels.isOn){
            specialAngleDrawnView.angleB = specialAngleDrawnView.angleG + CGFloat.random(in: .pi/6 ... .pi*5/6)
        }
        else{
            specialAngleDrawnView.angleB =  specialAngleDrawnView.angleA
        }
                

        //crate tasks
        switch currentStateOfTask {
        case 0:
            //if angle has to be identified
            taskLabel.text = NSLocalizedString("special:task:identify", comment: "special:task:identify")
            for button in listOfButtons{
                button.alpha = 1
            }
            button4.alpha = 0
            currentSpecialAngle = Int.random(in: 0...3)
            checkButton.isEnabled = false
        case 1:
            //if angle has to be adjusted
            currentSpecialAngle = Int.random(in: 0...3)
            taskLabel.text = dictionaryOfAngleTasks[currentSpecialAngle]
            for button in listOfButtons{
                button.alpha = 0
            }
            checkButton.isEnabled = true
        default:
            //for teamwork
            taskLabel.text = NSLocalizedString("special:task:together", comment: "special:task:together")
            for button in listOfButtons{
                button.alpha = 1
                button.isEnabled = false
            }
            checkButton.isEnabled = false
        }
        
        
        //draw all the quarter
        let firstColoredQuarter = Int.random(in: 0...7)
        for quarter in listOfQuarters{
            switch currentStateOfTask {
            case 0:
                //if angle has to be identified
                quarter.isUserInteractionEnabled = false
                quarter.isSelected = false
                switch currentSpecialAngle {
                case 0:
                    quarter.isGiven = (quarter.index == firstColoredQuarter || quarter.index == calcVerticalAngleIndex(angleIndex: firstColoredQuarter))
                case 1:
                    quarter.isGiven = (quarter.index == firstColoredQuarter || quarter.index == calcAdjacentAngleIndex(angleIndex: firstColoredQuarter))
                case 2:
                    quarter.isGiven = (quarter.index == firstColoredQuarter || quarter.index == calcCorrespondingAngleIndex(angleIndex: firstColoredQuarter))
                case 3:
                    quarter.isGiven = (quarter.index == firstColoredQuarter || quarter.index == calcAlternateAngleIndex(angleIndex: firstColoredQuarter))
                default:
                    quarter.isGiven = false
                }
            case 1:
                //if angle has to be drawn
                quarter.isUserInteractionEnabled = !(quarter.index == firstColoredQuarter)
                quarter.isGiven = (quarter.index == firstColoredQuarter)
                quarter.isSelected = false
            default:
                //for teamwork
                quarter.isUserInteractionEnabled = true
                quarter.isGiven = false
                quarter.isSelected = false
            }
            
            var refPoint = specialAngleDrawnView.pointG1
            var refAngle = specialAngleDrawnView.angleA
            if(quarter.index>=4){
                refPoint = specialAngleDrawnView.pointG2
                refAngle = specialAngleDrawnView.angleB
            }
            
            var startAngle = specialAngleDrawnView.angleG
            if(quarter.index%2==1){
                startAngle = refAngle
            }
            if(2...3 ~= quarter.index%4){
                startAngle += .pi
            }
            
            var endAngle = specialAngleDrawnView.angleG
            if(quarter.index%2==0){
                endAngle = refAngle
            }
            if(1...2 ~= quarter.index%4){
                endAngle += .pi
            }
            
            quarter.renewPosition(center: refPoint, startAngle: startAngle, endAngle: endAngle)
        }

        specialAngleDrawnView.setNeedsDisplay()
    }
    
    
    //create point with random position
    func createRandomPoint(angle: CGFloat) -> CGPoint{
        let smallWidth = specialAngleView.frame.width/4
        let largeWidth = min(3*specialAngleView.frame.width/8,(specialAngleView.frame.width-150)/2)
        let point = CGPoint(x: specialAngleView.frame.width/2+CGFloat.random(in: smallWidth...largeWidth)*cos(angle), y: specialAngleView.frame.height/2+CGFloat.random(in: smallWidth...largeWidth)*sin(angle))
        return point
    }
    
    
    //enable or disable buttons, when two angles are selected
    func enableButtonsByQuarters(){
        var selectedQuartersIndizes : [Int] = []
        for quarter in listOfQuarters.filter({$0.isSelected}) {
            selectedQuartersIndizes += [quarter.index]
        }
        if(selectedQuartersIndizes.count == 2){
            for button in listOfButtons{
                button.isEnabled = true
            }
        }
        else{
            for button in listOfButtons{
                button.isEnabled = false
                button.isSelected = false
            }
            if(currentStateOfTask == 2){
                checkButton.isEnabled = false
            }
        }
    }
    
    //update view, when Mac-window is resized
    override func viewWillTransition(to size: CGSize, with coordinator: UIViewControllerTransitionCoordinator) {
        specialAngleDrawnView.setNeedsDisplay()
    }

}
