//
//  TouchPoint.swift
//  Winkel
//
//  Created by Heiko Etzold on 22.01.18.
//  Copyright © 2018 Heiko Etzold. All rights reserved.
//

import UIKit

class TouchPoint: UIView, UIPointerInteractionDelegate {

    let touchPointRadius = CGFloat(40)
    var index = Int(0)
    var visibleTouchPoint = UIView()
    
    var connectedViewController = UIViewController()
    
    
    init(position: CGPoint, connectedViewController: UIViewController, small: Bool) {
        super.init(frame: CGRect(origin: position, size: CGSize(width: 2*touchPointRadius, height: 2*touchPointRadius)))
        self.center = position
        self.connectedViewController = connectedViewController
        createTouchPoint(small: small)
        
        
        //set pointer (for trackpad/mouse)
        if #available(iOS 13.4, *) {
            let pointerInteraction = UIPointerInteraction(delegate: self)
            self.addInteraction(pointerInteraction)
        }
    }
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    //do great things, when pointer enters touch point
    @available(iOS 13.4, *)
    func pointerInteraction(_ interaction: UIPointerInteraction, styleFor region: UIPointerRegion) -> UIPointerStyle? {
        var pointerStyle: UIPointerStyle? = nil
        
        if let interactionView = interaction.view {
            let targetedPreview = UITargetedPreview(view: interactionView)
            if(!isBegan){
                pointerStyle = UIPointerStyle(effect: UIPointerEffect.highlight(targetedPreview))
            }
        }
        return pointerStyle
    }
    

    //make a touchpoint with a visible point inside
    func createTouchPoint(small: Bool){
        self.backgroundColor = UIColor.clear
        self.layer.cornerRadius = touchPointRadius

        visibleTouchPoint.backgroundColor = tintColor.withAlphaComponent(0.5)
        self.addSubview(visibleTouchPoint)
        
        var scale : CGFloat {
            if(small){
                return 0.25
            }
            return 1
        }
        
        visibleTouchPoint.translatesAutoresizingMaskIntoConstraints = false
        visibleTouchPoint.centerXAnchor.constraint(equalTo: self.centerXAnchor).isActive = true
        visibleTouchPoint.centerYAnchor.constraint(equalTo: self.centerYAnchor).isActive = true
        visibleTouchPoint.widthAnchor.constraint(equalTo: self.widthAnchor, multiplier: scale).isActive = true
        visibleTouchPoint.heightAnchor.constraint(equalTo: self.heightAnchor, multiplier: scale).isActive = true
    }
    
        
    var isBegan = Bool(false)
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        isBegan = true
        scaleView(factor: 1.25)
    }


    var isMoving = Bool(false)
    override func touchesMoved(_ touches: Set<UITouch>, with event: UIEvent?) {
        isMoving = true
        isBegan = false


        //for exploring angles only
        if let vc = connectedViewController as? ExploreAngleViewController{

            vc.exploreDrawnAngleView.lockButton.alpha = 0

            let pointInfinityOrthogonalToA = homogCross(firstVector: [vc.lineA[1],-vc.lineA[0],0], secondVector: [0,0,1])

            //normal use
            if(!vc.exploreDrawnAngleView.isLockButtonLocked || self.index>=4){
                self.center += (touches.first?.location(in: self))!-(touches.first?.previousLocation(in: self))!
                if(!isInArea(point: self.center)){
                    self.center -= (touches.first?.location(in: self))!-(touches.first?.previousLocation(in: self))!
                }
            }
            
    
            //when lines are parallel (locked) and a point on them is moving
            if(vc.exploreDrawnAngleView.isLockButtonLocked && self.index<4){
               
                var otherPoint : TouchPoint {
                    switch self.index {
                    case 0:
                        return vc.listOfTouchPoints[1]
                    case 1:
                        return vc.listOfTouchPoints[0]
                    case 2:
                        return vc.listOfTouchPoints[3]
                    default:
                        return vc.listOfTouchPoints[2]
                    }
                }
                var otherLine : [CGFloat] {
                    switch self.index {
                    case 0:
                        return vc.lineB
                    case 1:
                        return vc.lineB
                    case 2:
                        return vc.lineA
                    default:
                        return vc.lineA
                    }
                }
                
                var sameLine : [CGFloat] {
                    switch self.index {
                    case 0:
                        return vc.lineA
                    case 1:
                        return vc.lineA
                    case 2:
                        return vc.lineB
                    default:
                        return vc.lineB
                    }
                }
                
                //if it's the only moving point on this line, only allow to move on line
                if(!otherPoint.isMoving){
                    let potentialNewCenter = self.center + (touches.first?.location(in: self))!-(touches.first?.previousLocation(in: self))!

                    let pointInfinityOrthogonalToSelfLine = homogCross(firstVector: [sameLine[1],-sameLine[0],0], secondVector: [0,0,1])
                    let lineOrthogonalToSelfLineThroughSelf = homogCross(firstVector: [potentialNewCenter.x,potentialNewCenter.y,1], secondVector: pointInfinityOrthogonalToSelfLine)

                    let newSelfCenter = pointByLines(firstLine: lineOrthogonalToSelfLineThroughSelf, secondLine: sameLine)
                    //only do this, if you keep in the super view
                    if(isInArea(point: newSelfCenter)){
                        self.center = newSelfCenter
                    }
                }
                    
                //if both points are moving, allow parallel translation
                else{
                    let potentialNewCenter = self.center + (touches.first?.location(in: self))!-(touches.first?.previousLocation(in: self))!

                    let midPoint = 0.5*(potentialNewCenter+otherPoint.center)
                    let pointInfinityOrthogonalToOtherLine = homogCross(firstVector: [otherLine[1],-otherLine[0],0], secondVector: [0,0,1])
                    let pointInfinityInDirectionOtherLine = [otherLine[1],-otherLine[0],0]
                    let lineParallelToOtherLineThroughMidPoint = homogCross(firstVector: pointInfinityInDirectionOtherLine, secondVector: [midPoint.x,midPoint.y,1])

                    let lineOrthogonalToNewLineThroughSelf = homogCross(firstVector: [potentialNewCenter.x,potentialNewCenter.y,1], secondVector: pointInfinityOrthogonalToOtherLine)
                    let newSelfCenter = pointByLines(firstLine: lineParallelToOtherLineThroughMidPoint, secondLine: lineOrthogonalToNewLineThroughSelf)
                    
                    let lineOrthogonalToNewLineThroughOtherPoint = homogCross(firstVector: [otherPoint.center.x,otherPoint.center.y,1], secondVector: pointInfinityOrthogonalToOtherLine)
                    let newOtherCenter = pointByLines(firstLine: lineParallelToOtherLineThroughMidPoint, secondLine: lineOrthogonalToNewLineThroughOtherPoint)

                    //only do this, if you keep in the super view
                    if(isInArea(point: newSelfCenter) && isInArea(point: newOtherCenter)){
                        self.center = newSelfCenter
                        otherPoint.center = newOtherCenter
                    }
                }
            }

            
            //show lockButton, when lines are nearly parallel or when button is locked
            let refQuarter = vc.listOfQuarters.filter({$0.index%4==0})
            if(vc.exploreDrawnAngleView.isLockButtonLocked || round(refQuarter[0].ownSize*180/(.pi)) == round(refQuarter[1].ownSize*180/(.pi))){
                let lineOrthogonalToAThroughA1 = homogCross(firstVector: [vc.listOfTouchPoints[0].center.x,vc.listOfTouchPoints[0].center.y,1], secondVector: pointInfinityOrthogonalToA)
                let pointOnLineB = pointByLines(firstLine: lineOrthogonalToAThroughA1, secondLine: vc.lineB)
                vc.exploreDrawnAngleView.orthogonalPoint = pointOnLineB
                vc.exploreDrawnAngleView.lockButton.center = 0.5*(pointOnLineB+vc.listOfTouchPoints[0].center)
                vc.exploreDrawnAngleView.lockButton.alpha = 1
            }
            
            //update everything
            vc.updateQuarters()
            vc.exploreDrawnAngleView.setNeedsDisplay()
        }
    }

    
    //check, if point is in super view
    func isInArea(point: CGPoint) -> Bool{
        if let vc = connectedViewController as? ExploreAngleViewController{
            
            var leftBorder = CGFloat(0)
            var rightBorder = vc.exploreDrawnAngleView.frame.width
            var topBorder = CGFloat(0)
            var bottomBorder = UITabBarController().tabBar.frame.origin.y
            
            if #available(iOS 11.0, *) {
                leftBorder = vc.exploreDrawnAngleView.safeAreaInsets.left
                topBorder = vc.exploreDrawnAngleView.safeAreaInsets.top
                rightBorder -= -vc.exploreDrawnAngleView.safeAreaInsets.right
                bottomBorder = vc.exploreDrawnAngleView.frame.height-vc.exploreDrawnAngleView.safeAreaInsets.bottom
            }
            
            return (point.x-visibleTouchPoint.frame.width/2>leftBorder && point.y-visibleTouchPoint.frame.height/2>topBorder && point.x+visibleTouchPoint.frame.width/2<rightBorder && point.y+visibleTouchPoint.frame.height/2<bottomBorder)
        }
        return false
    }

    
    override func touchesEnded(_ touches: Set<UITouch>, with event: UIEvent?) {
        scaleView(factor: 0.8)
        isMoving = false
        isBegan = false
    }
        
    
    
    override func layoutSubviews() {
        self.layer.cornerRadius = self.bounds.width/2
        super.layoutSubviews()
        visibleTouchPoint.layer.cornerRadius = visibleTouchPoint.bounds.width/2
    }
    
    
    func scaleView(factor: CGFloat){
        self.center += CGPoint(x: (1-factor)*self.bounds.width/2, y: (1-factor)*self.bounds.height/2)
        self.frame.size.width *= factor
        self.frame.size.height *= factor
    }
    
}
